#!/bin/bash
# vim: dict=/usr/share/beakerlib/dictionary.vim cpt=.,w,b,u,t,i,k
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   runtest.sh of 
#   Description: Test nss_ldap with primary/secondary server (soft failure).
#   Author: David Spurek <dspurek@redhat.com>
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   Copyright (c) 2011 Red Hat, Inc. All rights reserved.
#
#   This copyrighted material is made available to anyone wishing
#   to use, modify, copy, or redistribute it subject to the terms
#   and conditions of the GNU General Public License version 2.
#
#   This program is distributed in the hope that it will be
#   useful, but WITHOUT ANY WARRANTY; without even the implied
#   warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
#   PURPOSE. See the GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public
#   License along with this program; if not, write to the Free
#   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
#   Boston, MA 02110-1301, USA.
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

# Include Beaker environment
. /usr/bin/rhts-environment.sh
. /usr/lib/beakerlib/beakerlib.sh

# Affected authconfig files to be backed up
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
for file in /etc/pam.d/system-auth-ac \
        /etc/sssd \
        /etc/group \
        /etc/gshadow \
        /etc/hesiod.conf \
        /etc/krb.conf \
        /etc/krb5.conf \
        /etc/ldap.conf \
        /etc/libuser.conf \
        /etc/login.defs \
        /etc/nsswitch.conf \
        /etc/nslcd.conf \
        /etc/pam_smb.conf \
        /etc/passwd \
        /etc/samba/smb.conf \
        /etc/shadow \
        /etc/sysconfig/authconfig \
        /etc/sysconfig/network \
        /etc/yp.conf \
        /etc/krb5.conf \
        /var/kerberos/krb5kdc \
        /etc/krb5.keytab \
        ; do
    test -e $file && ConfigFiles="$ConfigFiles $file"
done

function do_ssh {
expect <<EOF
set timeout 20
spawn ssh $1@my-domain.com
expect "*(yes/no)?*" {send "yes\r"; exp_continue}
expect "$1@my-domain.com's password:" { send "$2\r" }
expect ".*$"
expect "$1@my-domain.com's password:" { exit 3 }
send "whoami\r"
expect "$1" { exit 0 }
    
exit 2
EOF
}



PACKAGES=( "nss_ldap"      \
        "expect"           \
        "authconfig"       \
        "openldap"         \
        "openldap-clients" \
        "openldap-servers" \
        "openssh-clients"  \
	"openldap-servers-overlays" )

BDN='cn=Manager,dc=my-domain,dc=com'
PWD='x'

#used base64 -id instead of only -d - BZ#719317
BASE64_OPTION="-id"

SLAPD_MASTER_PID=""
SLAPD_SLAVE_PID=""


rlJournalStart

      for P in ${PACKAGES[@]}; do
        rlCheckRpm $P
      done		
     
    rlPhaseStartSetup "General"

	rlServiceStop "ldap"
	rlServiceStop "nscd"

	rlRun "rlFileBackup $ConfigFiles" 0 "Backing up config files"
	rlFileBackup "/etc/hosts"
	rlFileBackup "/etc/openldap/cacerts/"  
	rlFileBackup "/etc/sysconfig/ldap"               
	rlRun "echo '127.0.0.1 my-domain.com' >> /etc/hosts" 0
	rlRun "mkdir /tmp/ldap-test-ldap-master" 0
	rlRun "mkdir /tmp/ldap-test-run-master" 0
	rlRun "mkdir /tmp/ldap-test-ldap-slave" 0
	rlRun "mkdir /tmp/ldap-test-run-slave" 0
	rlRun "chown ldap:ldap /tmp/ldap-test-run-master" 0
	rlRun "chown ldap:ldap /tmp/ldap-test-ldap-master" 0
	rlRun "chown ldap:ldap /tmp/ldap-test-run-slave" 0
	rlRun "chown ldap:ldap /tmp/ldap-test-ldap-slave" 0
	rlRun "chcon --reference=/var/run/openldap /tmp/ldap-test-run-master" 0
	rlRun "chcon --reference=/var/lib/ldap /tmp/ldap-test-ldap-master" 0
	rlRun "chcon --reference=/var/run/openldap /tmp/ldap-test-run-slave" 0
	rlRun "chcon --reference=/var/lib/ldap /tmp/ldap-test-ldap-slave" 0
	rlRun "cp -f ldap /etc/sysconfig/ldap" 0
	rlRun "mkdir /etc/openldap/certs/" 0
	rlRun "cp servercrt.pem /etc/openldap/certs/" 0
	rlRun "cp serverkey.pem /etc/openldap/certs/" 0
	rlRun "chown ldap:ldap /etc/openldap/certs/*" 0
	rlRun "chmod 644 /etc/openldap/certs/*" 0

        if [ -e /etc/openldap/cacerts/  ] ; then
                rlFileBackup "/etc/openldap/cacerts/" && rlRun "rm -rf /etc/openldap/cacerts/*" 0
        else
                mkdir /etc/openldap/cacerts/
        fi

	rlRun "cp cacert.pem /etc/openldap/cacerts/" 0
	rlRun "chown ldap:ldap /etc/openldap/cacerts/*" 0
	rlRun "chmod 644 /etc/openldap/cacerts/*" 0
	rlRun "cacertdir_rehash /etc/openldap/cacerts/" 0
	rlRun "semanage port -a -t ldap_port_t -p tcp 6360" 0
	rlRun "semanage port -a -t ldap_port_t -p tcp 6361" 0

	rlRun "slapadd -l data.ldif -f slapd-master.conf" 0
	rlRun "slapadd -l data.ldif -f slapd-slave.conf" 0
	rlRun "slapadd -l add.ldif -f slapd-master.conf" 0
	rlRun "slapadd -l add.ldif -f slapd-slave.conf" 0

	rlRun "chown -R ldap:ldap /tmp/ldap-test-run-master" 0
	rlRun "chown -R ldap:ldap /tmp/ldap-test-ldap-master" 0
	rlRun "chown -R ldap:ldap /tmp/ldap-test-run-slave" 0
	rlRun "chown -R ldap:ldap /tmp/ldap-test-ldap-slave" 0

	rlRun "restorecon -Rv /etc/"

    rlPhaseEnd
	
    rlPhaseStartTest "Test nss_ldap collaboration two directory servers without TLS/SSL" 
    
        ( /usr/sbin/slapd               \
	    -u ldap                     \
	    -h "ldap://my-domain.com:6360" \
	    -d 257 -f slapd-master.conf >slapd-master.log 2>&1) &

	sleep 5 

        ( /usr/sbin/slapd               \
	    -u ldap                     \
	    -h "ldap://my-domain.com:6361" \
	    -d 257 -f slapd-slave.conf >slapd-slave.log 2>&1) &

	sleep 5

	SLAPD_MASTER_PID=`ps ax | grep "slapd.*6360" | grep -v grep | awk '{print $1}'`
	SLAPD_SLAVE_PID=`ps ax | grep "slapd.*6361" | grep -v grep | awk '{print $1}'`

	if [ $SLAPD_MASTER_PID"X" = "X" ]; then
	    rlLog "master is not running :-("
	else
	    rlLog "master is running :-)"

	    if [ $SLAPD_SLAVE_PID"X" = "X" ]; then
		rlLog "slave is not running :-("
	    else
		rlLog "slave is running :-)"

		rlRun "authconfig --enableshadow --disablecache --enablemd5 --enableldap --enableldapauth \
		 --ldapbasedn=dc=my-domain,dc=com --enablelocauthorize --enablemkhomedir --updateall" 0

		cat /etc/ldap.conf | grep -v "host" | grep -v "uri"> /tmp/ldap_nohost.conf
		cp /tmp/ldap_nohost.conf /etc/ldap.conf

		rlRun "cp -f nsswitch.conf /etc/nsswitch.conf" 0

		rlRun "echo 'uri ldap://my-domain.com:6361 ldap://my-domain.com:6360' >> /etc/ldap.conf" 0
		rlRun "echo 'bind_timelimit 5' >> /etc/ldap.conf" 0 
		rlLog "soft failure secondary/primary (i.e. directory 'hang')  (/etc/ldap.conf - uri ldap://my-domain.com:6361 ldap://my-domain.com:6360)"
		rlLog "Try that ssh user2 function before stop slave server"
		rlRun "do_ssh user2 x" 0

		rlRun "kill -s SIGSTOP $SLAPD_SLAVE_PID" 0 "Kill SIGSTOP slave openldap server (master should be used)"
		sleep 5
		SUSPEND_SLAPD=""
		SUSPEND_SLAPD=`ps ax | grep "slapd.*6361" | grep -v grep | grep Tl`
		if [ $? ]; then
			rlLog "slave is stopped/suspend :-)"
	   		rlRun "do_ssh user2 x" 0
		else
			rlLog "slave is running :-("
		fi
	    fi
	fi
    rlPhaseEnd

  rlPhaseStartTest "SELinux"
    rlRun "ausearch -m AVC -ts recent > /tmp/out 2>&1" 0-255
    rlRun "grep '<no matches>' /tmp/out" 0 "There should not be any SELinux AVC"
    cat /tmp/out && rm -f /tmp/out
  rlPhaseEnd
		
    rlPhaseStartCleanup
     
	rlRun "rm -rf /tmp/ldap-* /etc/openldap/cacerts/* /etc/openldap/certs/" 0
	rm -f /tmp/ldap_nohost.conf
        rlFileRestore
	rlServiceRestore "ldap"
	rlServiceRestore "nscd"
	rlRun "semanage port -d -t ldap_port_t -p tcp 6360" 0
	rlRun "semanage port -d -t ldap_port_t -p tcp 6361" 0

    rlPhaseEnd

rlJournalPrintText

rlJournalEnd
